#! /usr/bin/env python3

import logging
import httpx
import typer
import json

from snyk_tags import __app_name__, __version__

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}", 'Content-Type': 'application/json'} 
    )

# Apply tags to a specific project
def apply_attributes_to_project(
    client: httpx.Client, org_id: str, project_id: str, criticality: list, environment: list, lifecycle: list, project_name: str
) -> tuple:
    attribute_data = {
        "criticality": criticality,
        "environment": environment,
        "lifecycle": lifecycle
    }
    
    req = client.post(f"org/{org_id}/project/{project_id}/attributes", data=json.dumps(attribute_data))
    
    attribute_data = typer.style(attribute_data, bold=True, fg=typer.colors.MAGENTA)
    if req.status_code == 200:
        logging.info(f"Successfully added {attribute_data} attributes to Project: {project_name}.")
    
    if req.status_code == 422:
        logging.warning(f"Data {attribute_data} cannot be processed, make sure you have written the correct values (refer to help or Readme) and that they are in low caps.")
    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}.")
    if req.status_code == 500:
        logging.error(f"Error {req.status_code}: Internal Server Error. Please contact eric.fernandez@snyk.io.")
    
    return req.status_code, req.json()

#
def apply_attributes_to_projects(token: str, org_ids: list, name: str, criticality: list, environment: list, lifecycle: list) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            for project in projects.get("projects"):
                if project["name"].startswith(name):
                    apply_attributes_to_project(
                            client=client, org_id=org_id, project_id=project["id"], criticality=criticality, environment=environment, lifecycle=lifecycle, project_name=project["name"]
                        )

# Coloured variables for output
repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)
crit = typer.style("critical, high, medium, low", bold=True, fg=typer.colors.MAGENTA)
enviro = typer.style("frontend, backend, internal, 	external, mobile, saas, onprem, hosted, distributed", bold=True, fg=typer.colors.MAGENTA)
life = typer.style("production, development, sandbox", bold=True, fg=typer.colors.MAGENTA)

# Collection command to apply the attributes to the collection
@app.command(help=f"Apply attributes to a project collection\n\n Use the name you see in the collection as the name: name={repoexample} to set the attributes for everything under that repo, container or CLI import")
def collection(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify the Organization ID where you want to apply the attributes"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  collectionName: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the project collection, for example {repoexample}"
        ),  criticality: str = typer.Option(
            ..., # Default value of comamand
            help=f"Criticality attribute: {crit}"
        ),  environment: str = typer.Option(
            ..., # Default value of comamand
            help=f"Environment attribute: {enviro}"
        ),  lifecycle: str = typer.Option(
            ..., # Default value of comamand
            help=f"Lifecycle attribute: {life}"
        )
    ):
    typer.secho(f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {collectionName} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
    apply_attributes_to_projects(token,[org_id], collectionName, [criticality], [environment], [lifecycle])


#! /usr/bin/env python3

import logging
import httpx
import typer

from rich import print

from snyk_tags import __app_name__, __version__

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )

# Get all organizations within a Group
def get_org_ids(token: str, group_id: str) -> list:
    org_ids = []
    with create_client(token=token) as client:
        req = client.get(f"group/{group_id}/orgs")
        if req.status_code == 404:
            logging.error(f"Group id: {group_id} is invalid. Error message: {req.json()}.")
        orgs = client.get(f"group/{group_id}/orgs").json()
        
        for org in orgs.get("orgs"):
            org_ids.append(org["id"])
    return org_ids

# Apply tags using the project tag API
def apply_tag_to_project(
    client: httpx.Client, org_id: str, project_id: str, tag: str, key: str, project_name: str
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }
    req = client.post(f"org/{org_id}/project/{project_id}/tags", data=tag_data)

    if req.status_code == 200:
        logging.info(f"Successfully added {tag} tags to Project: {project_name}.")
    if req.status_code == 422:
        logging.warning(f"{tag} tag is already applied for Project: {project_name}.")
    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}.")
    return req.status_code, req.json()

def apply_tags_to_projects(token: str, org_ids: list, types: list, tag: str, key: str) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            for project in projects.get("projects"):
                for type in types:
                    if project["type"] == type:
                        logging.debug(
                            apply_tag_to_project(
                                client=client, org_id=org_id, project_id=project["id"], tag=tag, key=key, project_name=project["name"]
                            )
                        )

# SAST Command
sasttypes = typer.style("\n sast", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Code tag to Snyk Code projects")
def sast(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  sastType: str = typer.Option(
            "", # Default value of comamand
            help=f"Type of Snyk Code projects to apply tags to: {sasttypes}"
        )
    ):

    org = []
    type = []
    if org_id == '' or None:
        if sastType == '' or None:
            sastType = ["sast"]
            typer.secho(f"\nAdding the Code tag to {sastType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, sastType, tag='Code', key='Product')
        else:
            type.append(sastType)
            typer.secho(f"\nAdding the Code tag to {sastType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, type, tag='Code', key='Product')
    else:
        if sastType == '' or None:
            sastType = ["sast"]
            typer.secho(f"\nAdding the Code tag to {sastType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, sastType, tag='Code', key='Product')
        else:
            type.append(sastType)
            typer.secho(f"\nAdding the Code tag to {sastType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org_ids, type, tag='Code', key='Product')



# IaC Command
iactypes = typer.style("\n terraformconfig\n terraformplan\n k8sconfig\n helmconfig\n cloudformationconfig\n armconfig", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply IaC tag to Snyk IaC projects")
def iac(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  iacType: str = typer.Option(
            "", # Default value of comamand
            help=f"Type of Snyk IaC projects to apply tags to: {iactypes}"
        )
    ):

    org = []
    type = []
    if org_id == '' or None:
        if iacType == '' or None:
            iacType=["terraformconfig", "terraformplan", "k8sconfig", "helmconfig", "cloudformationconfig", "armconfig"]
            typer.secho(f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, iacType, tag='IaC', key='Product')
        else:
            type.append(iacType)
            typer.secho(f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org, type, tag='IaC', key='Product')
    else:
        if iacType == '' or None:
            iacType=["terraformconfig", "terraformplan", "k8sconfig", "helmconfig", "cloudformationconfig", "armconfig"]
            typer.secho(f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, iacType, tag='IaC', key='Product')
        else:
            type.append(iacType)
            typer.secho(f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, type, tag='IaC', key='Product')



# SCA Command
scatypes = typer.style("\nmaven\n npm\n nuget\n gradle\n pip\n yarn\n gomodules\n rubygems\n composer\n sbt\n golangdep\n cocoapods\n poetry\n govendor\n cpp\n yarn-workspace\n hex\n paket\n golang", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Open Source tag to Snyk Open Source projects")
def sca(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand,
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  scaType: str = typer.Option(
            "", # Default value of comamand
            help=f"Type of Snyk Open Source projects to apply tags to (default:all): {scatypes}"
        )
    ):

    org = []
    type = []
    if org_id == '' or None:
        if scaType == '' or None:
            scaType = ["maven","npm","nuget", "gradle", "pip", "yarn", "gomodules", "rubygems", "composer", "sbt", "golangdep", "cocoapods", "poetry", "govendor", "cpp", "yarn-workspace", "hex", "paket", "golang"]
            typer.secho(f"\nAdding the Open Source tag to {scaType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, scaType, tag='Open Source', key='Product')
        else:
            type.append(scaType)
            typer.secho(f"\nAdding the Open Source tag to {scaType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, type, tag='Open Source', key='Product')
    else:
        if scaType == '' or None:
            scaType = ["maven","npm","nuget", "gradle", "pip", "yarn", "gomodules", "rubygems", "composer", "sbt", "golangdep", "cocoapods", "poetry", "govendor", "cpp", "yarn-workspace", "hex", "paket", "golang"]
            typer.secho(f"\nAdding the Open Source tag to {scaType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, scaType, tag='Open Source', key='Product')
        else:
            type.append(scaType)
            typer.secho(f"\nAdding the Open Source tag to {scaType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, type, tag='Open Source', key='Product')



# Container Command
containertypes = typer.style("\n dockerfile\n apk\n deb\n rpm\n linux", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Container tag to Snyk Container projects")
def container(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  containerType: str = typer.Option(
            "", # Default value of comamand
            help=f"Type of Snyk Container projects to apply tags to (default:all): {containertypes}"
        )
    ):

    org = []
    type =[]
    if org_id == '' or None:
        if containerType == '' or None:
            containerType= ["dockerfile", "apk", "deb", "rpm", "linux"]
            typer.secho(f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, containerType, tag='Container', key='Product')
        else:
            type.append(containerType)
            typer.secho(f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI", bold=True)
            org_ids = get_org_ids(token, group_id)
            apply_tags_to_projects(token, org_ids, type, tag='Container', key='Product')
    else:
        if containerType == '' or None:
            containerType= ["dockerfile", "apk", "deb", "rpm", "linux"]
            typer.secho(f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, containerType, tag='Container', key='Product')
        else:
            type.append(containerType)
            typer.secho(f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI", bold=True)
            org.append(org_id)
            apply_tags_to_projects(token, org, type, tag='Container', key='Product')

# Custom Command
@app.command(help="Apply custom tags to the preferred project type")
def custom(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  projectType: str = typer.Option(
            ..., # Default value of comamand
            help=f"Type of Snyk project to apply tags to (choose one): {sasttypes}, {containertypes}, {scatypes}, {iactypes}"
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):

    typer.secho(f"\nAdding the tag key {tagKey} and tag value {tagValue} to {projectType} projects in Snyk for easy filtering via the UI", bold=True)
    org = []
    type =[]
    type.append(projectType)
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, type, tagValue, tagKey)
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, type, tagValue, tagKey)




#! /usr/bin/env python3

import typer

from typing import Optional
from snyk_tags import __app_name__, __version__, list, collection, tag, attribute

snyk = typer.style("snyk-tags", bold=True)
snykcmd = typer.style("snyk-tags tag --help", bold=True, fg=typer.colors.MAGENTA)
snykcmd2 = typer.style("snyk-tags attribute --help", bold=True, fg=typer.colors.MAGENTA)
snykcmd3 = typer.style("snyk-tags collection --help", bold=True, fg=typer.colors.MAGENTA)

app = typer.Typer(
    help=f"{snyk} helps you filter Snyk projects by adding product tags and attributes to projects per product or collection of projects\n\n To start using it try running:\n\n - {snykcmd} \n\n - {snykcmd2}  \n\n - {snykcmd3}",
    add_completion=False,
    no_args_is_help=True
    )
app.add_typer(tag.app, name="tag", help="Apply product/custom tags based on project type, select which tag to apply")
app.add_typer(list.app, name="list", help="List all project types of a Snyk product and all attribute types")
app.add_typer(collection.app, name="collection", help="Apply custom tags to all projects within a collection e.g Git repo")
app.add_typer(attribute.app, name="attribute", help="Apply attributes to all projects within a collection e.g Git repo")


def main():
    return True

def _version_callback(value: bool) -> None:
    if value:
        typer.secho(f"snyk-tags v{__version__}", bold=True)
        raise typer.Exit()

@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snyk_tags']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.55,<2.0',
 'colorama>=0.4.5,<0.5.0',
 'httpx>=0.23.0,<0.24.0',
 'rich>=10.11.0',
 'shellingham>=1.4.0,<2.0.0',
 'typer>=0.5.0']

entry_points = \
{'console_scripts': ['snyk-tags = snyk_tags.tags:app']}

setup_kwargs = {
    'name': 'snyk-tags',
    'version': '0.6.0',
    'description': 'Tool designed to add tags in bulk to Snyk projects',
    'long_description': "# Snyk Tags Tool\n\nSnyk Tags is a CLI tool with three purposes:\n\n- Help filter Snyk projects by product type by adding product tags across a Snyk Group or Organization - using ```snyk-tags tag```\n- Help filter Snyk projects by applying tags to a collection of projects (for example a git repo like **snyk-labs/nodejs-goof**) - using ```snyk-tags collection```\n- Help filter Snyk projects by applying attributes to a collection of projects (for example a git repo like **snyk-labs/nodejs-goof**) - using ```snyk-tags attribute```\n\n### snyk-tags tag\n\n```snyk-tags tag``` is a CLI tool that uses the Snyk Project Tag API to assign tags in bulk to Snyk projects based on the product.\n\n```snyk-tags tag``` will update all projects of the specified product within a Snyk Group or Organization with the product's tag.\n\nYou can also specify a custom tag for the specific project types.\n\n### snyk-tags collection\n\n```snyk-tags collection``` uses the Snyk Project Tag API to assign tags to all projects within a collection. A collection encompasses one or more projects in Snyk, for example:\n\n- **snyk-labs/nodejs-goof** is a collection from a git import\n- **library/httpd** is a collection from a container import\n- **/snyk-labs/nodejs-goof** is a collection from a CLI import\n\nYou can also use ```snyk-tags collection``` to add your Github repository code owner as a tag to an imported repo with  ```snyk-tags collection github```\n\n[List all project types](#list-of-all-project-types)\n\n### snyk-tags attribute\n\n```snyk-tags attribute``` uses the Snyk Project Attribute API to assign attributes to all projects within a collection. A collection encompasses one or more projects in Snyk, for example:\n\n- **snyk-labs/nodejs-goof** is a collection from a git import\n- **library/httpd** is a collection from a container import\n- **/snyk-labs/nodejs-goof** is a collection from a CLI import\n\n[List all possible attributes](#list-of-all-attributes)\n\n### Viewing results\n\nOnce you run ```snyk-tags```, go into the UI, naviagate to the projects page and find the tags filter or attribute filter options on the left-hand menu. Select the tag/attribute you have applied and you will see all projects associated.\n\n## Installation and requirements\n\n### Requirements\n\nRequires Python version above 3.6\n\n### Installation\n\nTo install the simplest way is to use pip:\n\n```bash\npip install snyk-tags\n```\n\nAlternatively you can clone the repo and then run the following commands:\n\n```python\npoetry install # To install dependencies\npython -m snyk-tags # To run snyk-tags\n```\n\n## Examples\n\nI want to filter all my Snyk Code projects to the whole Snyk Group:\n\n``` bash\nsnyk-tags tag sast --group-id=abc --token=abc\n```\n\nI want to filter all my ```npm``` Snyk Open Source projects within a specific Snyk Organization:\n\n``` bash\nsnyk-tags tag sca --scatype=npm --org-id=abc --token=abc\n```\n\nI want to filter all projects within my ```snyk-labs/nodejs-goof``` repo by ```project:snyk```\n\n``` bash\nsnyk-tags collection tag --collectionname=snyk-labs/nodejs-goof --org-id=abc --token=abc --tagkey=project --tagvalue=snyk\n```\n\nI want to add attributes to all projects within my ```snyk-labs/python-goof``` repo. The attributes are ```critical, production, backend```\n\n``` bash\nsnyk-tags attribute collection  --collectionname=snyk-labs/python-goof --org-id=abc --token=abc --criticality=critical --environment=backend --lifecycle=production\n```\n\nI want to filter all projects within my ```snyk-labs/nodejs-goof``` repo by ```Owner:EricFernandezSnyk```\n\n``` bash\nsnyk-tags collection github --reponame=snyk-labs/nodejs-goof --org-id=abc --snyktoken=abc --githubtoken=abc\n```\n\n## Types of projects and attributes\n\n### List of all project types\n\n|       Snyk IaC       | Snyk Open Source | Snyk Container | Snyk Code |\n|:--------------------:|:----------------:|:--------------:|:---------:|\n|    terraformconfig   |       maven      |   dockerfile   |    sast   |\n|     terraformplan    |        npm       |       apk      |           |\n|       k8sconfig      |       nuget      |       deb      |           |\n|      helmconfig      |      gradle      |       rpm      |           |\n| cloudformationconfig |        pip       |      linux     |           |\n|       armconfig      |       yarn       |                |           |\n|                      |     gomodules    |                |           |\n|                      |     rubygems     |                |           |\n|                      |     composer     |                |           |\n|                      |        sbt       |                |           |\n|                      |     golangdep    |                |           |\n|                      |     cocoapods    |                |           |\n|                      |      poetry      |                |           |\n|                      |     govendor     |                |           |\n|                      |        cpp       |                |           |\n|                      |  yarn-workspace  |                |           |\n|                      |        hex       |                |           |\n|                      |       paket      |                |           |\n|                      |      golang      |                |           |\n\n### List of all attributes\n\n| Criticality          | Environment | Lifecycle       |\n|:--------------------:|:-----------:|:---------------:|\n|       critical       |   frontend  |    production   |\n|         high         |   backend   |   development   |\n|        medium        |   internal  |     sandbox     |\n|          low         |   external  |                 |\n|                      |    mobile   |                 |\n|                      |     saas    |                 |\n|                      |    onprem   |                 |\n|                      |    hosted   |                 |\n|                      | distributed |                 |\n",
    'author': 'EricFernandezSnyk',
    'author_email': 'eric.fernandez@snyk.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/EricFernandezSnyk/snyk-tags-tool',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
